import pygame

import matplotlib.cm as cmx
import matplotlib.colors as colors

class Reward:

    def __init__(self):
        self.dict = dict()
        self.max_reward     = 10.0
        self.default_reward = 0.0
        self.min_reward     = 0.0

        self.set_color_map(cmx.get_cmap('jet'))

    def set_color_map(self,cm):
        self.cNorm      = colors.Normalize(vmin=self.min_reward, vmax=self.max_reward)
        self.scalarMap  = cmx.ScalarMappable(norm=self.cNorm, cmap=cm)

    def clear(self):
        self.dict.clear()

    def add(self,state,value,done):
        """ Adds a reward value and outcome to a state
                Args:
                    state       (int)   : index of state
                    value       (float) : value of reward for this state
                    done        (bool)  : if this is a terminal state
        """
        self.dict.update({state : (value,done)})

    def R(self,state):
        if state in self.dict.keys():
            return self.dict[state][0], self.dict[state][1]
        else:
            return self.default_reward, False


    def draw(self,screen,state2grid,grid2pixel_pos,scale,cell_size):

        for state in self.dict:
            i,j = state2grid(state)

            #                                 left-right                    up-down
            sq_bl_pos = grid2pixel_pos(i - cell_size / 2.0 + (0.02 * cell_size),j + cell_size / 2.0 - (0.02 * cell_size) )

            length = scale * (cell_size - 0.01 * cell_size)

            c = self.scalarMap.to_rgba(self.dict[state][0])
            pygame.draw.rect(screen, (c[0] * 255,c[1] * 255,c[2] * 255), (sq_bl_pos[0],sq_bl_pos[1],length,length),0)



def get_default_reward():
    reward = Reward()
    reward.default_reward   = 0
    reward.min_reward       = 0
    reward.max_reward       = 10
    reward.add(state=99,value=10,done=True)
    return reward



import pygame
import sys

class Keyboard:

    def __init__(self):
        pass

    def get_action(self):

        while True:
            for event in pygame.event.get():
                if event.type == pygame.QUIT: sys.exit()
                if event.type == pygame.KEYDOWN:
                    if event.key == pygame.K_ESCAPE: sys.exit()
                    if event.key == pygame.K_RIGHT:
                        return 0
                    if event.key == pygame.K_LEFT:
                        return 1
                    if event.key == pygame.K_UP:
                        return 2
                    if event.key == pygame.K_DOWN:
                        return 3



import sys, pygame
import math
import numpy as np

class Grid:

    def __init__(self,length,num_bins=10):
        self.pts        = np.linspace(0,length,num_bins,False)
        self.cell_size  = self.pts[1] - self.pts[0]
        self.pts        = self.pts[1:]

        self.num_bins   = num_bins
        self.length     = length

        # compute all states
        self.states = np.empty([self.num_bins * self.num_bins,3])
        idx = 0
        for i in range(0,self.num_bins):
            for j in range(0,self.num_bins):
                self.states[idx,:] = np.array([self.grid2state(i,j),i,j])
                idx=idx+1

    def init_render(self,screen,scale,sq_pos):

        # colors
        self.black          = (0,0,0)
        self.scale          = scale

        self.width          = screen.get_width()
        self.height         = screen.get_height()
        self.offset         = (self.cell_size / 2.0) * self.scale
        self.sq_pos         = sq_pos

        default_font        = pygame.font.get_default_font()
        self.font_renderer  = pygame.font.Font(default_font, 20)

        self.pts_px     = []
        for x in self.pts:
            start_pos       = [self.scale * (x - self.length / 2.0),  self.scale * (0 - self.length / 2.0)]
            end_pos         = [self.scale * (x - self.length / 2.0),  self.scale * (self.length - self.length / 2.0)]
            start_pos_px    = self.world2pixel_pos(x=start_pos[0],y=start_pos[1])
            end_pos_px      = self.world2pixel_pos(x=end_pos[0],y=end_pos[1])
            self.pts_px.append([start_pos_px,end_pos_px])
        self.pts_py     = []
        for y in self.pts:
            start_pos       = [self.scale * (0 - self.length / 2.0),          self.scale * (y - self.length / 2.0)]
            end_pos         = [self.scale * (self.length - self.length / 2.0),     self.scale * (y - self.length / 2.0)]
            start_pos_py    = self.world2pixel_pos(x=start_pos[0],y=start_pos[1])
            end_pos_py      = self.world2pixel_pos(x=end_pos[0],y=end_pos[1])
            self.pts_py.append([start_pos_py,end_pos_py])

    def pt2grid(self,x,y):
        """ x and y are in the grid's frame of reference which is at
            the bottom left corner of the square.
        """
        if x < 0: x = 0
        if x >= self.length: x = self.length - self.cell_size
        if y < 0: y = 0
        if y >= self.length: y = self.length - self.cell_size

        x = round( (float(x) / float(self.length)) * float(self.num_bins))
        y = round( (float(y) / float(self.length)) * float(self.num_bins))

        return np.array([x,y])

    def grid2state(self,i,j):
        """ i and j are integer coordinates, returns a state coordinate """
        return int(j * self.num_bins + i)

    def state2grid(self,state):
        return float(state) % float(self.num_bins), int(float(state) / float(self.num_bins))

    def world2pixel_pos(self,x,y):
        return map(int,[x + self.width/2.0, -y + self.height/2.0])

    def grid2pixel_pos(self,x,y):
        return map(int,[x * self.scale + self.sq_pos + self.offset, -y * self.scale + self.height - self.sq_pos - self.offset])

    def world2pixel_orient(self,orientation):
        return orientation + math.pi / 2.0

    def _draw_states(self,screen):
        for row in self.states:
            text = self.font_renderer.render(str(int(row[0])), 1, self.black)
            text_w = text.get_rect().width
            text_h = text.get_rect().height
            pos = self.grid2pixel_pos(row[1],row[2])
            screen.blit(text, [pos[0] - text_w/2,pos[1] - text_h/2])

    def draw(self,screen):
        for px in self.pts_px:
            pygame.draw.line(screen, self.black, px[0], px[1], 2)
        for py in self.pts_py:
            pygame.draw.line(screen, self.black, py[0], py[1], 2)

        self._draw_states(screen=screen)



# -*- coding: utf-8 -*-
"""
@author: Guillaume de Chambrier

    A simple square world

"""

import math
import gym
from gym import spaces
from gym.utils import seeding
import numpy as np

from square_world.square_world import SquareWorld
from square_world.reward import Reward

class SquareEnv(gym.Env):

    metadata = {
        'render.modes': ['human', 'rgb_array'],
        'video.frames_per_second': 30
    }

    def __init__(self):
        self._seed()
        self.reset()

        self.square_world       = SquareWorld()
        self.action_space       = spaces.Discrete(4)
        self.observation_space  = self.square_world.get_observation_space()

        # control command to agent
        self._u = np.array([0,0])

    def set_reward(self,reward):
        self.square_world.reward = reward

    def _seed(self, seed=None):
        self.np_random, seed = seeding.np_random(seed)
        return [seed]

    def _step(self, action):
        assert self.action_space.contains(action), "%r (%s) invalid" % (action, type(action))
        if action == 0:
            # right
            self._u[0]      =  1
            self._u[1]      =  0
        elif action == 1:
            # left
            self._u[0]      = -1
            self._u[1]      =  0
        elif action == 2:
            # up
            self._u[0]      =  0
            self._u[1]      =  1
        elif action == 3:
            # down
            self._u[0]      =  0
            self._u[1]      = -1
        else:
            self._u[0]      =  0
            self._u[1]      =  0

        pos, state, reward, done = self.square_world.update(self._u)

        return state, reward, done, {}

    def _reset(self):
        pass

    def _render(self, mode='human', close=False):
        self.square_world.render(close)



import unittest
import gym

from gym_square.envs.square_env import SquareEnv
from time import sleep
import numpy as np
import matplotlib.cm as cmx


class TestLeftRightEnv(unittest.TestCase):

    def test_env(self):

        env = SquareEnv()
        env.reset()

        env.square_world.set_agent_state(55)

        cm = cmx.get_cmap('brg')
        env.square_world.reward.set_color_map(cm)


        for _ in range(1):
            #env.render()

            action = env.action_space.sample()
            observation, reward, done, info = env.step(action)

            print 'act:  ', action
            print 'obs:  ', observation
            print 'rew:  ', reward
            print 'done: ', done
            print ' '

            if done:
                print 'Episode Finished'
                break

            sleep(0.001)

        return True


if __name__ == '__main__':
    unittest.main()



import sys, pygame
import math
import numpy as np


from grid import Grid
from agent import Agent
import reward as re


class SquareWorld:

    def __init__(self,num_bins=10,reward=re.get_default_reward()):

        self.length     = 10
        self._bRender   = False

        self.reward     = reward
        self.grid       = Grid(length=self.length,num_bins=num_bins)
        self.agent      = Agent(radius=(self.grid.cell_size / 2.0))

    def init_render(self):
        if self._bRender == False:
            pygame.init()
            size = 800, 800
            self.screen = pygame.display.set_mode(size)
            pygame.font.init()
            default_font = pygame.font.get_default_font()

            min_size            = min((self.screen.get_width(),self.screen.get_height()))
            self.sq_px_length   = min_size - (2 * 0.1 * min_size)
            self.sq_pos         = 0.1 * min_size
            self.border_color   = (0,0,0)
            self.border_width   = 5
            self.scale          = self.sq_px_length / self.length

            #  initialise grid rendering
            self.grid.init_render(screen=self.screen,scale=self.scale,sq_pos=self.sq_pos)

            # initialise agent rendering
            self.agent.init_render(scale=self.scale)

            self._bRender        = True

    def get_observation_space(self):
        """ All Discrete states id with indicies (i,j)
            Returns
                (numpy.ndarray) : N x 3
                                 [:,0]   : state id
                                 [:,1:2] : indices (i,j)
        """
        return self.grid.states

    def set_agent_state(self,state):
        i,j = self.grid.state2grid(state)
        self.agent.pos[0] = i
        self.agent.pos[1] = j
        self.agent.pos_tmp = self.agent.pos[:]

    def set_reward(self,reward):
        self.reward = reward

    def update(self,u):
        """ Upates the position of the agent in the square world

            Args:
                u       (numpy.ndarray) : 2D motion vector
            Returns:
                state   (numpy.ndarray) : current state
                reward  (float)         : numerical reward
                done    (bool)          : if episode is finished or not
        """

        # agent motion check that it is valid
        pos_u      = self.agent.pos + u
        pos        = self.grid.pt2grid(x=pos_u[0],y=pos_u[1])
        state      = self.grid.grid2state(i=pos[0],j=pos[1])

        self.agent.update(pos)

        # get the reward
        reward, done = self.reward.R(state=state)

        return pos, state, reward, done


    def render(self,close=False):

        if self._bRender and close == False:
            for event in pygame.event.get():
                if event.type == pygame.QUIT: sys.exit()

            self.screen.fill( (255, 255, 255) )

            self.reward.draw(self.screen,self.grid.state2grid,self.grid.grid2pixel_pos,self.scale,self.grid.cell_size)

            pygame.draw.rect(self.screen, self.border_color, (self.sq_pos,self.sq_pos,self.sq_px_length,self.sq_px_length), self.border_width)
            self.grid.draw(self.screen)
            self.agent.draw(self.screen,self.grid.grid2pixel_pos)

            pygame.display.flip()
        elif self._bRender and close:
            self._bRender = False
            pygame.quit()
        elif self._bRender == False and close == False:
            self.init_render()



import math
import pygame
import numpy as np
import sys

class Agent:

    def __init__(self,radius,pos=[0,0],theta=0):
        self.theta      = theta
        self.pos        = np.array(pos)
        self.pos_tmp    = np.array(pos)
        self.radius     = radius
        self.heading    = np.array([math.cos(theta), math.sin(theta)])


        self._end_pos   = np.array([0,0])
        self._disp      = np.array([0,0])

    def init_render(self,scale):
        self.agent_radius_px  = int(self.radius * scale)


    def update(self,pos):
        if np.sum(pos - self.pos) != 0:
            self.pos_tmp    = self.pos[:]
            self.pos        = pos

            self._disp      = self.pos - self.pos_tmp
            self._disp      = self._disp / ( np.sqrt(np.sum(self._disp**2)) + 0.00001 )
            self.theta      = math.atan2(self._disp[1],self._disp[0])

    def draw(self,screen,grid2pixel_pos):

        pos_agent_px      = grid2pixel_pos(x=self.pos[0],y=self.pos[1])

        self.heading      = 0.5 * np.array([ math.cos(self.theta),math.sin(self.theta)])
        self._end_pos     = self.pos + self.heading

        end_pos_px        = grid2pixel_pos(self._end_pos[0],self._end_pos[1])

        pygame.draw.circle(screen, (255,165,0), pos_agent_px, self.agent_radius_px, 0)
        pygame.draw.line(screen, (0,0,0), pos_agent_px, end_pos_px, 5)



from setuptools import setup
from setuptools import find_packages

setup(name='gym_square',
      version='0.0.1',
      author='Guillaume de Chambrier',
      author_email='chambrierg@gmail.com',
      description='A simple square world environment for openai/gym',
      packages=find_packages(),
      url='https://github.com/gpldecha/gym-square',
      license='MIT',
      install_requires=['gym']
)


